@@ -98,6 +98,7 @@
 #define CURLE_OUT_OF_MEMORY_FIRST 0x070100 /* Added in 7.1 */
 #define CURLE_PARTIAL_FILE_FIRST 0x070100 /* Added in 7.1 */
 #define CURLE_PEER_FAILED_VERIFICATION_FIRST 0x071101 /* Added in 7.17.1 */
+#define CURLE_QUIC_CONNECT_ERROR_FIRST 0x074500 /* Added in 7.69.0 */
 #define CURLE_QUOTE_ERROR_FIRST 0x071100 /* Added in 7.17.0 */
 #define CURLE_RANGE_ERROR_FIRST 0x071100 /* Added in 7.17.0 */
 #define CURLE_READ_ERROR_FIRST 0x070100 /* Added in 7.1 */
@@ -312,10 +313,10 @@
 #define CURLMOPT_CHUNK_LENGTH_PENALTY_SIZE_FIRST 0x071e00 /* Added in 7.30.0 */
 #define CURLMOPT_CONTENT_LENGTH_PENALTY_SIZE_FIRST 0x071e00 /* Added in 7.30.0 */
 #define CURLMOPT_MAXCONNECTS_FIRST 0x071003 /* Added in 7.16.3 */
+#define CURLMOPT_MAX_CONCURRENT_STREAMS_FIRST 0x074300 /* Added in 7.67.0 */
 #define CURLMOPT_MAX_HOST_CONNECTIONS_FIRST 0x071e00 /* Added in 7.30.0 */
 #define CURLMOPT_MAX_PIPELINE_LENGTH_FIRST 0x071e00 /* Added in 7.30.0 */
 #define CURLMOPT_MAX_TOTAL_CONNECTIONS_FIRST 0x071e00 /* Added in 7.30.0 */
-#define CURLMOPT_MAX_CONCURRENT_STREAMS_FIRST 0x074300 /* Added in 7.67.0 */
 #define CURLMOPT_PIPELINING_FIRST 0x071000 /* Added in 7.16.0 */
 #define CURLMOPT_PIPELINING_SERVER_BL_FIRST 0x071e00 /* Added in 7.30.0 */
 #define CURLMOPT_PIPELINING_SITE_BL_FIRST 0x071e00 /* Added in 7.30.0 */
@@ -329,6 +330,7 @@
 #define CURLMSG_NONE_FIRST 0x070906 /* Added in 7.9.6 */
 #define CURLM_ADDED_ALREADY_FIRST 0x072001 /* Added in 7.32.1 */
 #define CURLM_BAD_EASY_HANDLE_FIRST 0x070906 /* Added in 7.9.6 */
+#define CURLM_BAD_FUNCTION_ARGUMENT_FIRST 0x074500 /* Added in 7.69.0 */
 #define CURLM_BAD_HANDLE_FIRST 0x070906 /* Added in 7.9.6 */
 #define CURLM_BAD_SOCKET_FIRST 0x070f04 /* Added in 7.15.4 */
 #define CURLM_CALL_MULTI_PERFORM_FIRST 0x070906 /* Added in 7.9.6 */
@@ -337,14 +339,15 @@
 #define CURLM_OK_FIRST 0x070906 /* Added in 7.9.6 */
 #define CURLM_OUT_OF_MEMORY_FIRST 0x070906 /* Added in 7.9.6 */
 #define CURLM_RECURSIVE_API_CALL_FIRST 0x073b00 /* Added in 7.59.0 */
-#define CURLM_WAKEUP_FAILURE_FIRST 0x074400 /* Added in 7.68.0 */
 #define CURLM_UNKNOWN_OPTION_FIRST 0x070f04 /* Added in 7.15.4 */
+#define CURLM_WAKEUP_FAILURE_FIRST 0x074400 /* Added in 7.68.0 */
+#define CURLOPT_FIRST 0x074500 /* Added in 7.69.0 */
 #define CURLOPTTYPE_FUNCTIONPOINT_FIRST 0x070100 /* Added in 7.1 */
 #define CURLOPTTYPE_LONG_FIRST 0x070100 /* Added in 7.1 */
 #define CURLOPTTYPE_OBJECTPOINT_FIRST 0x070100 /* Added in 7.1 */
 #define CURLOPTTYPE_OFF_T_FIRST 0x070b00 /* Added in 7.11.0 */
-#define CURLOPTTYPE_STRINGPOINT_FIRST 0x072e00 /* Added in 7.46.0 */
 #define CURLOPTTYPE_SLISTPOINT_FIRST 0x074102 /* Added in 7.65.2 */
+#define CURLOPTTYPE_STRINGPOINT_FIRST 0x072e00 /* Added in 7.46.0 */
 #define CURLOPT_ABSTRACT_UNIX_SOCKET_FIRST 0x073500 /* Added in 7.53.0 */
 #define CURLOPT_ACCEPTTIMEOUT_MS_FIRST 0x071800 /* Added in 7.24.0 */
 #define CURLOPT_ACCEPT_ENCODING_FIRST 0x071506 /* Added in 7.21.6 */
@@ -466,6 +469,7 @@
 #define CURLOPT_MAIL_AUTH_FIRST 0x071900 /* Added in 7.25.0 */
 #define CURLOPT_MAIL_FROM_FIRST 0x071400 /* Added in 7.20.0 */
 #define CURLOPT_MAIL_RCPT_FIRST 0x071400 /* Added in 7.20.0 */
+#define CURLOPT_MAIL_RCPT_ALLLOWFAILS_FIRST 0x074500 /* Added in 7.69.0 */
 #define CURLOPT_MAXAGE_CONN_FIRST 0x074100 /* Added in 7.65.0 */
 #define CURLOPT_MAXCONNECTS_FIRST 0x070700 /* Added in 7.7 */
 #define CURLOPT_MAXFILESIZE_FIRST 0x070a08 /* Added in 7.10.8 */
@@ -683,6 +687,7 @@
 #define CURLPROTO_IMAPS_FIRST 0x071400 /* Added in 7.20.0 */
 #define CURLPROTO_LDAP_FIRST 0x071304 /* Added in 7.19.4 */
 #define CURLPROTO_LDAPS_FIRST 0x071304 /* Added in 7.19.4 */
+#define CURLPROTO_MQTT_FIRST 0x074700 /* Added in 7.71.0 */
 #define CURLPROTO_POP3_FIRST 0x071400 /* Added in 7.20.0 */
 #define CURLPROTO_POP3S_FIRST 0x071400 /* Added in 7.20.0 */
 #define CURLPROTO_RTMP_FIRST 0x071500 /* Added in 7.21.0 */
@@ -754,6 +759,7 @@
 #define CURLSSLOPT_NO_REVOKE_FIRST 0x072c00 /* Added in 7.44.0 */
 #define CURLSSLSET_NO_BACKENDS_FIRST 0x073800 /* Added in 7.56.0 */
 #define CURLSSLSET_OK_FIRST 0x073800 /* Added in 7.56.0 */
+#define CURLSSLOPT_REVOKE_BEST_EFFORT_FIRST 0x074600 /* Added in 7.70.0 */
 #define CURLSSLSET_TOO_LATE_FIRST 0x073800 /* Added in 7.56.0 */
 #define CURLSSLSET_UNKNOWN_BACKEND_FIRST 0x073800 /* Added in 7.56.0 */
 #define CURLUE_BAD_HANDLE_FIRST 0x073e00 /* Added in 7.62.0 */
@@ -795,8 +801,8 @@
 #define CURLU_DISALLOW_USER_FIRST 0x073e00 /* Added in 7.62.0 */
 #define CURLU_GUESS_SCHEME_FIRST 0x073e00 /* Added in 7.62.0 */
 #define CURLU_NON_SUPPORT_SCHEME_FIRST 0x073e00 /* Added in 7.62.0 */
-#define CURLU_NO_DEFAULT_PORT_FIRST 0x073e00 /* Added in 7.62.0 */
 #define CURLU_NO_AUTHORITY_FIRST 0x074300 /* Added in 7.67.0 */
+#define CURLU_NO_DEFAULT_PORT_FIRST 0x073e00 /* Added in 7.62.0 */
 #define CURLU_PATH_AS_IS_FIRST 0x073e00 /* Added in 7.62.0 */
 #define CURLU_URLDECODE_FIRST 0x073e00 /* Added in 7.62.0 */
 #define CURLU_URLENCODE_FIRST 0x073e00 /* Added in 7.62.0 */
@@ -805,8 +811,9 @@
 #define CURLVERSION_FOURTH_FIRST 0x071001 /* Added in 7.16.1 */
 #define CURLVERSION_NOW_FIRST 0x070a00 /* Added in 7.10 */
 #define CURLVERSION_SECOND_FIRST 0x070b01 /* Added in 7.11.1 */
-#define CURLVERSION_THIRD_FIRST 0x070c00 /* Added in 7.12.0 */
+#define CURLVERSION_SEVENTH_FIRST 0x074600 /* Added in 7.70.0 */
 #define CURLVERSION_SIXTH_FIRST 0x074200 /* Added in 7.66.0 */
+#define CURLVERSION_THIRD_FIRST 0x070c00 /* Added in 7.12.0 */
 #define CURL_CHUNK_BGN_FUNC_FAIL_FIRST 0x071500 /* Added in 7.21.0 */
 #define CURL_CHUNK_BGN_FUNC_OK_FIRST 0x071500 /* Added in 7.21.0 */
 #define CURL_CHUNK_BGN_FUNC_SKIP_FIRST 0x071500 /* Added in 7.21.0 */
@@ -953,7 +960,6 @@
 #define CURL_VERSION_CONV_FIRST 0x070f04 /* Added in 7.15.4 */
 #define CURL_VERSION_CURLDEBUG_FIRST 0x071306 /* Added in 7.19.6 */
 #define CURL_VERSION_DEBUG_FIRST 0x070a06 /* Added in 7.10.6 */
-#define CURL_VERSION_ESNI_FIRST 0x074300 /* Added in 7.67.0 */
 #define CURL_VERSION_GSSAPI_FIRST 0x072600 /* Added in 7.38.0 */
 #define CURL_VERSION_GSSNEGOTIATE_FIRST 0x070a06 /* Added in 7.10.6 */
 #define CURL_VERSION_HTTP2_FIRST 0x072100 /* Added in 7.33.0 */
@@ -977,5 +983,6 @@
 #define CURL_WAIT_POLLIN_FIRST 0x071c00 /* Added in 7.28.0 */
 #define CURL_WAIT_POLLOUT_FIRST 0x071c00 /* Added in 7.28.0 */
 #define CURL_WAIT_POLLPRI_FIRST 0x071c00 /* Added in 7.28.0 */
+#define CURL_WIN32_FIRST 0x074500 /* Added in 7.69.0 */
 #define CURL_WRITEFUNC_PAUSE_FIRST 0x071200 /* Added in 7.18.0 */
 #define CURL_ZERO_TERMINATED_FIRST 0x073800 /* Added in 7.56.0 */
